"""
All custom Balcony App definitions must be exported here as well.
"""

from .iam_trusted_entity_helper import IAMTrustedEntities
from .terraform_iam_import_commands import TerraformIAMImportCommandGenerator
from .service_summary import BalconyServiceSummarizer


from balcony import BaseBalconyApp, BalconyAWS
from typer.main import Typer
import typer
from typing import Optional
import jmespath


balcony = BalconyAWS()
def get_role_names():
    role_data = balcony.get_service('iam').read('Role')
    expr = 'ListRoles[*].Roles[*].RoleName[]'
    role_names = jmespath.search(expr, role_data)
    return role_names

def generate_import_command(tf_aws_type, tf_resource, import_id):
    return f"terraform import {tf_aws_type}.{tf_resource} {import_id}"
      
def generate_role_import_commands():
    cmmds = []
    role_names = get_role_names()
    for i, role_name in enumerate(role_names):
        cmmds.append(generate_import_command('aws_iam_role', f"Role{str(i+1)}", role_name))
    return cmmds

class TerraformIAMImportCommandGenerator(BaseBalconyApp,
        author="og", 
        app_name="terraform-import-iam", 
        description="Generates terraform import commands",
        tags=('terraform', 'import')
    ):
    
    def __init__(self, *args, **kwargs) -> None:
        pass        

    def get_data(self, *args, **kwargs) -> dict:
        return generate_role_import_commands
    
    def get_cli_app(self, *args, **kwargs) -> typer.main.Typer:
        app = typer.Typer(no_args_is_help=True)
        
        @app.command('role')
        def _role_tf_importer(
            service: Optional[str] = typer.Argument(None, show_default=False,help='argument name'),
        ):
            for role_import_cmd in generate_role_import_commands():
                typer.echo(role_import_cmd)
        return app
                
    





from balcony import BaseBalconyApp, BalconyAWS
from typer.main import Typer
import typer
from typing import Optional
import jmespath


class IAMTrustedEntities(BaseBalconyApp,
        author="og", 
        app_name="iam-trusted-entities", 
        description="List the Account Ids that roles have trusted entity relation with.",
        tags=('iam', 'role', 'trusted-entity')
    ):
    
    def __init__(self, *args, **kwargs) -> None:
        pass
      
    def get_data(self, *args, **kwargs) -> dict:
        balcony = BalconyAWS()
        role_data = balcony.get_service('iam').read('Role')
        expr = 'ListRoles[*].Roles[*].AssumeRolePolicyDocument[].Statement[*].Principal[].AWS'
        return jmespath.search(expr, role_data)
    
    def get_cli_app(self, *args, **kwargs) -> typer.main.Typer:
        app = typer.Typer(no_args_is_help=True)
        
        @app.command('hey')
        def _hey_command(
            service: Optional[str] = typer.Argument(None, show_default=False,help='argument name'),
        ):
            typer.echo(service)
            typer.echo(self.get_data())
            typer.echo('-'*20)
            
        return app
                
    





from balcony import BaseBalconyApp, BalconyAWS, get_rich_console
from typer.main import Typer
import typer
from typing import Optional
import jmespath
import random
import time
from rich.layout import Layout
from rich.live import Live
from rich.table import Table
from rich.panel import Panel



console = get_rich_console()

class BalconyServiceSummarizer(BaseBalconyApp,
        author="og", 
        app_name="service-summarizer", 
        description="Prints a summary of the given Balcony service.",
        tags=('balcony', 'service-node')
    ):
    
    def __init__(self, *args, **kwargs) -> None:
        pass
      
    def get_data(self, *args, **kwargs) -> dict:
        service=kwargs.get('service')
        balcony = BalconyAWS()
        service_node = balcony.get_service(service)

        read_only_function_count = 0
        generated_resource_nodes = []
        for rn in service_node.get_resource_nodes():
            operation_names = rn.get_operation_names() 
            generated_resource_nodes.append({
                'name': rn.name,
                'operation_names': operation_names
            })
            read_only_function_count += len(operation_names)
        return {
            'read_only_function_count':read_only_function_count,
            'resource_nodes': generated_resource_nodes
        } 
       
    def get_cli_app(self, *args, **kwargs) -> typer.main.Typer:
        app = typer.Typer(no_args_is_help=True)
        
        
        def kayan_yazi(_list, window=5):
            length = len(_list)
            until = length - window
            for i in range(until+1):
                yield _list[i:i+window]
        
        def generate_live_render(data):
            nonlocal self
            resource_node_names = [r.get('name') for r in data.get('resource_nodes')]
            for ky in kayan_yazi(resource_node_names):
                str_ky = '\n'.join(ky)
                layout = Layout(Panel(ky), name="main")
                yield layout


        @app.callback(invoke_without_command=True)
        def _command(
            service: str = typer.Argument(None, show_default=False,help='AWS Service Name'),
        ):
            balcony = BalconyAWS()
            available_services = balcony.get_available_service_node_names()
            selected_service = random.choice(available_services)
            data = self.get_data(service=selected_service)
            print(data)
            with Live(selected_service, refresh_per_second=2) as live:
                for rendered in generate_live_render(data):
                    live.update(rendered)
                    time.sleep(0.4)
                    
        return app
                
    





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['balconyapp', 'balconyapp.app']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'balconyapp',
    'version': '0.0.30',
    'description': 'Balcony App template package',
    'long_description': '# balcony-app-template\nA template repository for creating new Balcony Apps.\n',
    'author': 'Oguzhan Yilmaz',
    'author_email': 'oguzhanylmz271@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
